"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateComponentTemplate = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const find_module_1 = require("@schematics/angular/utility/find-module");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
function generateComponentTemplate(options) {
    return () => {
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files'), [
            (0, schematics_1.applyTemplates)(Object.assign({ classify: core_1.strings.classify, dasherize: core_1.strings.dasherize }, options)),
            (0, schematics_1.move)((0, core_1.normalize)(`/${options.path}/${core_1.strings.dasherize(options.name)}`)),
        ]);
        options.type = 'Component';
        return (0, schematics_1.chain)([
            (0, schematics_1.externalSchematic)('@schematics/angular', 'component', {
                name: options.name,
                standalone: options.standalone,
                path: options.path,
                project: options.project,
                style: 'scss',
                skipImport: !options.isPageModule,
            }),
            options.isPageModule
                ? modifyAppToAddNewPage(options)
                : addDeclarationToModuleCpntsArr(options),
            (0, schematics_1.mergeWith)(templateSource, schematics_1.MergeStrategy.Overwrite),
        ]);
    };
}
exports.generateComponentTemplate = generateComponentTemplate;
let modifyAppToAddNewPage = (options) => {
    return (host) => {
        addScssToAppStyles(options, host);
        addRouteEntryToEnvArr(options, host);
        return host;
    };
};
function addDeclarationToModuleCpntsArr(options) {
    return (host) => {
        var _a;
        options.module = (0, find_module_1.findModuleFromOptions)(host, options);
        let modulePath = (_a = options.module) === null || _a === void 0 ? void 0 : _a.toString();
        let file = host.read(modulePath);
        if (!file) {
            throw new schematics_1.SchematicsException(`${modulePath} not found`);
        }
        let moduleFile = ts.createSourceFile(modulePath, file.toString(), ts.ScriptTarget.Latest, true);
        let cpntsVar = moduleFile.statements
            .filter(ts.isVariableStatement)
            .find((v) => {
            return (['components', 'cpnts'].includes(v.declarationList.declarations[0].name.getText()));
        });
        let cpntsArr = (0, ast_utils_1.findNodes)(cpntsVar, ts.SyntaxKind.ArrayLiteralExpression, 1)[0];
        let occurrencesCount = cpntsArr.elements.length;
        let text = cpntsArr.getFullText(moduleFile);
        let componentVar = core_1.strings.classify(options.name) + 'Component';
        let insertPos = cpntsArr.elements.pos;
        let finalVar = componentVar;
        if (occurrencesCount > 0) {
            let lastRouteLiteral = [...cpntsArr.elements].pop();
            let indentation = text.match(/\r?\n(\r?)\s*/) || [];
            let routeText = `${indentation[0] || ' '}${componentVar}`;
            insertPos = lastRouteLiteral.end;
            finalVar = `,${routeText}`;
        }
        let changes = [new change_1.InsertChange(modulePath, insertPos, finalVar)];
        // add import path
        addImportPath(options, modulePath, changes, moduleFile, componentVar, host);
        // add scss
        addScssToAppStyles(options, host);
        return host;
    };
}
let addRouteEntryToEnvArr = (options, host) => {
    var _a, _b;
    try {
        let targetPath = (0, core_1.normalize)('src/environments/environment.dev.ts');
        let targetFile = host.read(targetPath);
        if (!targetFile) {
            throw new schematics_1.SchematicsException(`${targetPath} not found`);
        }
        let targetSourceFile = ts.createSourceFile(targetPath, targetFile.toString(), ts.ScriptTarget.Latest, true);
        let devEnv = (_a = targetSourceFile === null || targetSourceFile === void 0 ? void 0 : targetSourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
            var _a;
            return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'DevEnv';
        });
        let navVar = devEnv.members.find((prop) => {
            var _a;
            return ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'nav';
        });
        let iife = navVar.getChildren().find((child) => {
            return ts.SyntaxKind.CallExpression === child.kind;
        });
        let expr = iife.getChildren().find((child) => {
            return ts.SyntaxKind.ParenthesizedExpression === child.kind;
        });
        let arrowFn0 = expr.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.ArrowFunction;
        });
        let block0 = arrowFn0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.Block;
        });
        let syntaxList0 = block0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.SyntaxList;
        });
        let var0 = syntaxList0.getChildren().find((child) => {
            let textBody = child.getFullText();
            return (child.kind === ts.SyntaxKind.VariableStatement &&
                textBody.includes('urls'));
        });
        let vardeclist0 = var0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.VariableDeclarationList;
        });
        let syntaxlist1 = vardeclist0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.SyntaxList;
        });
        let vardec0 = syntaxlist1.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.VariableDeclaration;
        });
        let objlit0 = vardec0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.ObjectLiteralExpression;
        });
        let syntaxList1 = objlit0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.SyntaxList;
        });
        let propAssign0 = syntaxList1.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.PropertyAssignment;
        });
        let objlit1 = propAssign0.getChildren().find((child) => {
            return child.kind === ts.SyntaxKind.ObjectLiteralExpression;
        });
        let route = '/' + options.route;
        let routeKey = ((_b = options.routeKey) !== null && _b !== void 0 ? _b : options.route);
        if (routeKey === "") {
            return;
        }
        let navObj = (0, ast_utils_1.findNodes)(objlit1, ts.SyntaxKind.ObjectLiteralExpression, 1)[0];
        let occurrencesCount = navObj.properties.length;
        let text = navObj.getFullText(targetSourceFile);
        let insertPos = navObj.properties.pos;
        let componentVar = `${routeKey}:"${route}"`;
        let finalVar = componentVar;
        if (occurrencesCount > 0) {
            let lastRouteLiteral = [
                ...navObj.properties,
            ].pop();
            let indentation = text.match(/\r?\n(\r?)\s*/) || [];
            let routeText = `${indentation[0] || ' '}${componentVar}`;
            insertPos = lastRouteLiteral.end;
            finalVar = `,${routeText}`;
        }
        let changes = [new change_1.InsertChange(targetPath, insertPos, finalVar)];
        (0, updateFile_1.wmlUpdateFile)(host, targetPath, changes);
    }
    catch (e) {
        return;
    }
};
function addScssToAppStyles(options, host) {
    try {
        let filePath = getFilePath(options);
        let componentLocation = (0, find_module_1.buildRelativePath)('src', filePath);
        let update0 = componentLocation.split('/');
        let cpntFileName = update0.at(-1);
        let cpntName = update0.at(-2);
        let cpntNames = update0.pop().split('.');
        cpntNames.pop();
        update0.shift();
        let cssLocation = update0.join('/');
        let phoneScss = cpntFileName + '.phone.scss';
        // let tabletScss = cpntFileName.split(".ts") + ".tablet.scss";
        let globalScss = cpntFileName + '.global.scss';
        let scssFiles = [globalScss, phoneScss].map((fileName) => {
            return [cssLocation, fileName].join('/');
        });
        let resultString = scssFiles.reduce((acc, x) => {
            return acc + '\n@import "' + x + '";';
        }, '// ' + cpntName);
        let appStylesPath = (0, core_1.normalize)('src/styles.scss');
        let appStylesFile = host.read(appStylesPath);
        if (!appStylesFile) {
            throw new schematics_1.SchematicsException(`${appStylesPath} not found`);
        }
        let appStylesFileString = appStylesFile.toString();
        let scssEndMarkerRegex = /\/\/ media-query-marker \(IMPT DONT DELETE THIS LINE\)/;
        let appStylesFileStringArray = appStylesFileString.split(scssEndMarkerRegex);
        if (appStylesFileStringArray.length !== 2) {
            return;
        }
        appStylesFileStringArray.splice(1, 0, '// media-query-marker (IMPT DONT DELETE THIS LINE)');
        appStylesFileStringArray.splice(1, 0, '\n\n');
        appStylesFileStringArray.splice(1, 0, resultString);
        let resultAppStylesFileString = appStylesFileStringArray.join('');
        host.overwrite(appStylesPath, resultAppStylesFileString);
    }
    catch (e) {
        return;
    }
}
function addImportPath(options, modulePath, changes, moduleFile, componentVar, host) {
    let filePath = getFilePath(options);
    let importPath = (0, find_module_1.buildRelativePath)(modulePath, filePath);
    changes.push((0, ast_utils_1.insertImport)(moduleFile, modulePath, componentVar, importPath));
    (0, updateFile_1.wmlUpdateFile)(host, modulePath, changes);
    return filePath;
}
function getFilePath(options) {
    return (`/${options.path}/` +
        (options.flat ? '' : core_1.strings.dasherize(options.name) + '/') +
        core_1.strings.dasherize(options.name) +
        (options.type ? '.' : '') +
        core_1.strings.dasherize(options.type));
}
//# sourceMappingURL=index.js.map